@@ -25,10 +25,12 @@
 
 mod head;
 
+pub use head::cursor::{Cursor, DoubleCursor};
+
 use {
     crate::head::{Iter, IterMut, ListHead},
     either::Either,
-    std::{iter::FromIterator, ptr},
+    std::{iter::FromIterator, ops::Not, ptr},
 };
 
 /// Create a list with provided elements.
@@ -164,7 +166,8 @@ impl<T> CircularList<T> {
     }
 
     /// Exchanges the place of the element at position `i` and the element at position `j`.
-    /// This operation has `O(n)` complexity.
+    /// This operation has `O(n)` complexity. For a constant time swap operation, see
+    /// [`double_cursor`](`Self::double_cursor`) and [`DoubleCursor::swap`].
     pub fn exchange(&mut self, i: usize, j: usize) {
         // Do nothing if list is empty
         if self.is_empty() {
@@ -313,6 +316,17 @@ impl<T> CircularList<T> {
         let len = self.len();
         self.iter_mut_forever().take(len)
     }
+
+    /// Returns some [`Cursor`] pointing to the first element of the list (if any).
+    pub fn cursor(&self) -> Option<Cursor<T>> {
+        self.is_empty().not().then(|| Cursor::from_list(self))
+    }
+
+    /// Returns some [`DoubleCursor`] where the 'a' and the 'b' parts are pointing both to the
+    /// first element of the list (if any).
+    pub fn double_cursor(&mut self) -> Option<DoubleCursor<T>> {
+        self.is_empty().not().then(|| DoubleCursor::from_list(self))
+    }
 }
 
 impl<T> Default for CircularList<T> {
@@ -522,4 +536,103 @@ mod tests {
         assert_eq!(a.len(), 0);
         assert_eq!(a.into_iter().collect::<Vec<i32>>(), &[]);
     }
+
+    #[test]
+    fn cursor_empty_list() {
+        assert_eq!(CircularList::<()>::default().cursor(), None)
+    }
+
+    #[test]
+    fn cursor() {
+        let list = list![1, 2, 3, 4, 5];
+        let mut c1 = list
+            .cursor()
+            .expect("A cursor should always be available on a non empty list");
+
+        assert_eq!(c1.value(), &1);
+
+        c1.move_prev();
+        assert_eq!(c1.value(), &5);
+
+        for _ in 0..5 {
+            c1.move_next();
+        }
+        assert_eq!(c1.value(), &5);
+
+        c1.move_next();
+        c1.move_next();
+        assert_eq!(c1.value(), &2);
+    }
+
+    #[test]
+    fn double_cursor_empty_list() {
+        assert!(matches!(
+            CircularList::<()>::default().double_cursor(),
+            None
+        ))
+    }
+
+    #[test]
+    fn double_cursor_swap() {
+        let mut list = list![1, 2, 3, 4, 5];
+        let mut dc = list
+            .double_cursor()
+            .expect("A cursor should always be available on a non empty list");
+
+        dc.move_next_b();
+        dc.swap();
+        assert_eq!(list.into_iter().collect::<Vec<i32>>(), &[2, 1, 3, 4, 5]);
+
+        let mut list = list![0];
+        let mut dc = list.double_cursor().unwrap();
+        dc.swap();
+        assert_eq!(list.into_iter().collect::<Vec<i32>>(), &[0]);
+    }
+
+    #[test]
+    fn double_cursor_move() {
+        let mut list = list![1, 2, 3, 4, 5];
+        let mut dc = list
+            .double_cursor()
+            .expect("A cursor should always be available on a non empty list");
+
+        dc.move_next_b();
+        dc.insert_a_after_b();
+        // This function is idempotent
+        dc.insert_a_after_b();
+        assert_eq!(list.into_iter().collect::<Vec<i32>>(), &[2, 1, 3, 4, 5]);
+    }
+
+    #[test]
+    fn double_cursor_sort() {
+        let mut list = list![3, 1, 8, 21, 5, 9, 12, 5, 2, 6, 6, 6, 13, 2, 17];
+        let len = list.len();
+        let mut dc = list
+            .double_cursor()
+            .expect("A cursor should always be available on a non empty list");
+
+        let mut min = *dc.value_a();
+        for i in 1..len {
+            dc.set_b_to_a();
+            dc.push_a();
+            for _ in i..len {
+                dc.move_next_a();
+                let val = *dc.value_a();
+                if val < min {
+                    min = val;
+                    dc.set_b_to_a();
+                }
+            }
+            dc.pop_a();
+            dc.insert_b_before_a();
+            if dc.a_is_b() {
+                dc.move_next_a();
+            }
+            min = *dc.value_a();
+        }
+        assert_eq!(
+            list.into_iter().collect::<Vec<i32>>(),
+            &[1, 2, 2, 3, 5, 5, 6, 6, 6, 8, 9, 12, 13, 17, 21]
+        );
+    }
 }
